######################################################################
## Constraint Function and Inverse Constraint Function for the mode ##
######################################################################

cfm <- function(x,a1=0,b1=1) ((b1-a1)/pi)*atan(x)+a1+(b1-a1)/2
mfc <- function(x,a1=0,b1=1)  tan((x-(a1+(b1-a1)/2))*pi/(b1-a1))


###########################################
## Reparameterized Beta density function ##
###########################################

betad <-
  function(
    x,    # evaluation points
    m,    # mode parameter
    h,    # variability parameter (denoted with "v" in the reference paper)
    a1=0, # lower bound of the support
    b1=1  # upper bound of the support
    ) {
    
    return(dbeta((x-a1)/(b1-a1), (m-a1)/(h*(b1-a1))+1, (b1-m)/(h*(b1-a1))+1) * 1/(b1-a1))
    
  }


##########################
## The mixture function ##
##########################

mixfunbet <- function(x,          # evaluation points
                      pesi,       # weights
                      parametri,  # parameters
                      a=0,        # lower bound of the support
                      b=1,        # upper bound of the support
                      G           # number of groups (k in the reference paper)
                      ) {
  
  # Case G=1
  if(G == 1){
    iniz <- betad(x,parametri[1],parametri[2],a1=a,b1=b) 
    return(iniz)
  }    
  
  # Case G>1
  else{
    fx   <- sapply(1:G, function(j) betad(x,parametri[j,1],parametri[j,2],a1=a,b1=b) )
    iniz <- fx %*% pesi
    return(iniz)
  }
  
}

################################################################################################
## Given the data, the function "kerder" provides:                                            ##
##           - kernel estimation of the density and of its first and second order derivatives ##
##           - Bump intervals (bumps) and their points of local maximum                       ##   
################################################################################################

kerder <- function(x=0,  # evaluation points
                   data, # observed data
                   h=0   # bandwidth
                   )
{
  
  n <- length(data)
  y <- density(data)
  if(h==0) h <- y$bw          # if h is not given, it is obtained through the function "density()"
  if(x==0) x <- y$x           # if x is not given, it is obtained through the function "density()"
  q <- length(x)
  f <- f1 <- f2 <- numeric(q) # functions are evaluated on all the q points x
  
  # Kernel estimation of the density (f) and its first and second order derivatives (f1 and f2)
  
  for(j in 1:q){
    f[j]  <-  1/(n*h*sqrt(2*pi))*sum(exp(-(x[j]-data)^2/(2*h^2)))
    f1[j] <- -1/(n*h^3*sqrt(2*pi))*sum((x[j]-data)*exp(-(x[j]-data)^2/(2*h^2)))
    f2[j] <-  1/(n*h^2*sqrt(2*pi))*sum(((x[j]-data)^2/h^2-1)*exp(-(x[j]-data)^2/(2*h^2)))
  }
  
  # Find the Bumps
  
  a <- numeric(q-1)
  b <- rep(NA,q-1)
  
  for(j in 1:(q-1)){
    a[j]                  <- approx(f2[j:(j+1)],x[j:(j+1)],0)$y
    if(!is.na(a[j])){b[j] <- 1/(n*h*sqrt(2*pi))*sum(exp(-(a[j]-data)^2/(2*h^2)))}
  }
  
  bump  <- a[which(a!="NA")]  # bump bounds: points on the x-axis where f2=0
  bumpy <- b[which(b!="NA")]  # estimated f in correspondence of the bump bounds
  bump  <- c(-Inf,bump,+Inf)  
  bumpy <- c(0,bumpy,0)        
  l     <- length(bump)        
  
  # Find the maximum value in each Bump
  
  maxim <- maximy <- maximxy <- NULL
  su <- 1
  bumpxy <- NULL
  
  for(i in 1:(l-1)){
    
    posx     <- which(x>=bump[i] & x<=bump[i+1])
    posmod   <- posx[f2[posx]<=0 & f[posx]==max(f[posx])]
    
    if(length(posmod!=0)){  
      maxim[su]  <- x[posmod]                                                    
      maximy[su] <- 1/(n*h*sqrt(2*pi))*sum(exp(-(maxim[su]-data)^2/(2*h^2)))   
      maximxy    <- rbind(maximxy,c(maxim[su],maximy[su]))
      bumpxy     <- rbind(bumpxy,cbind(bump[i:(i+1)],bumpy[i:(i+1)]))            
      su         <- su + 1
    }
    
  }
  
  return(list(f=f,
              f1=f1,
              f2=f2,
              x=x,
              h=h,
              bump=bump,        
              maxim=maxim,      
              bumpxy=bumpxy,    # points defining the bump bounds (first column) and correspondening values of "f"
              maximxy=maximxy   # points of maximum (first column) and maxima (second column) for "f" in all the bumps
              )
         )
  
}

###########################################################
## Given the number of mixture components,               ##
## the function "Modgroup" calculates the modes          ##
## and the corresponding values of the estimated density ##
###########################################################

Modgroup <-
  function(
    x,           # vector of data
    g,           # number of components of the beta mixture
    tol = 0.001, # tollerance for the stopping rule in the cicle for find the bandwidth
    maxit = 100  # maximum number of iterations to find the bandwidth
    ) {
    
    n <- length(x)
    
    h1  <- (max(x)-min(x))/1000
    h2  <- max(x)-min(x)
    
    # Algorithm to compute the bandwidth 
    
    for(i in 1:maxit){         
      hprov <- (h1+h2)/2
      inter <- kerder(data=x,h=hprov)
      gmean <- length(inter$maxim)
      
      if(gmean<=g) h2 <- hprov
      else         h1 <- hprov
      
      if(gmean==g & (h2-h1)/h1<tol) break
    }
    
    # Assigning bumps and modes according to the function kerder()
    
    hp         <- hprov
    modes      <- inter$maxim
    
    bump       <- inter$bump
    bumpxy     <- inter$bumpxy
    maximxy    <- inter$maximxy        
    
    return(list(
      modes = modes,      
      hp=hp,
      bump=bump,
      bumpxy=bumpxy,
      maximxy=maximxy
        )
      )
  }

###################################################################
## "Phi" calculates the starting parameters for the EM algorithm ##
###################################################################

Phi <-
  function(
    x,                          # vector of data
    g,                          # number of components of the beta mixture
    extr = c(0,1),              # bounds of the support of the beta components
    initialization = "random",  # inizialization technique: "random", "BH" (Bump Hunting), "manual"
    rr = 1,                     # number of replications in random initialization
    starting.values = NULL,     # manual inizialization (list with one vector of weights and one matrix of parameters to provide if initialization = "manual")
    maxit = 100,                # see the Modgroup() function
    tol = 0.001,                # see the Modgroup() function
    thre = 0.01                                  
    ) {
    
dati   <- x
n      <- length(x)
a      <- min(extr)
b      <- max(extr)
weight <- NULL 
param  <- matrix(,g,2) # m and v parameters of the beta mixture components
h      <- accam <- 0
   
if(initialization == "random"){
      
      prop    <- weight <- matrix(0,g,rr)
      Loglik  <- NULL
      paramet <- array(0,c(g,2,rr))
      
      for(l in 1:rr) {
        
        ## Weights ##
        
        prop[,l]   <- runif(g,0.2,1)
        weight[,l] <- prop[,l]/sum(prop[,l])
        
        
        ## Parameters ##
        
        paramet[,1,l] <- runif(g,a+0.1,b-0.1)
        paramet[,2,l] <- runif(g,0.01,0.5)
        
        Loglik[l] <- sum(log(
          sapply(1:n, function(i)
            mixfunbet(dati[i], G=g, parametri=paramet[,,l], pesi=weight[,l], a=a, b=b)
                 )
          ))
        
      }
      
      ind       <- which(Loglik==max(Loglik))
      weight    <- weight[,ind]
      param     <- paramet[,,ind]
    }
    
if(initialization == "manual"){
      
      weight <- starting.values[[1]] # numeric vector of size g
      param  <- starting.values[[2]] # (g x 2) matrix: m on the first column and v on the second one
      
    }
    
if(initialization == "BH"){
      
      G <- means <- variances <- acca <- accam <- weight <- NULL
      y <- sort(x)
      
      if(g == 1){   group <- Modgroup(x,g=g)
                    h     <- group$hp
                    
                    fiu <- function(x) {
                      fi <- 0
                      for(i in 1:n) {fi <- 1/(n*h*sqrt(2*pi))*exp(-(x-dati[i])^2/(2*h^2))+fi}
                      return(fi)
                    }
                    
                    lint   <- integrate(fiu,lower=a,upper=b)$value
                    
                    m      <- group$maximxy[1,1]
                    fm     <- group$maximxy[1,2]/lint
                    
                    fo     <- function (x) (((m-a)^((m-a)/(x*(b-a)))*(b-m)^((b-m)/(x*(b-a))))/((b-a)^((x+1)/x)*beta((m-a)/(x*(b-a))+1,(b-m)/(x*(b-a))+1)) -fm)^2
                    
                    acca   <- optimize(f=fo,interval=c(0,1000))$minimum
                    param  <- cbind(m,acca)
                    weight <- 1
      }
      
      else{
        
        group <- Modgroup(x,g=g)
        h     <- group$hp
        
        m     <- group$maximxy[,1]
        
        int   <- c(a,(m[1:(g-1)]+m[2:g])/2,b)
        
        for(j in 1:g){
          
          G[[j]]    <- y[which(y>=int[j] & y<int[j+1])]# generating g groups in the list G
          weight[j] <- length(G[[j]])/n
          ng        <- length(G[[j]])
          
          foia <- function(x,para=m,aaa=a,bbb=b) {
            logver <- 0
            for(i in 1:ng){
              logver <- log(betad(G[[j]][i],m[j],x,a1=aaa,b1=bbb)) + logver
            }
            return(-logver)
          }
          
          acca[j] <- optimize(f=foia,interval=c(thre,1))$minimum
        }
              
        param <- cbind(m,acca)
      }
    }
    
    return(
      list(
        pil = weight,  # vector containing the estimated weights
        theta = param, # (g x 2) matrix containing the estimated component parameters
        h=h,
        accam = accam  # value for h computed throught the means
        )
      )
  }

###################
### Beta Mixture ##
###################

betamix <-
  function(
    dat,                 # vector of data
    g,                   # number of components of the beta mixture
    extr = c(0,1),       # bounds of the support of beta distributions
    nx = 1000,           # maximum number of iterations
    rr = 1,              # number of random initializations
    eps = 0.00001,       # convergence limit (Note: if eps=-1, then the algorithm stops with nx)
    EMtype = "EM",       # EM-based variant: "EM", "CEM", "SEM"
    initial = "random",  # inizialization technique: "random", "BH", "manual"
    start.values = NULL, # starting values (see the Phi() function)
    soglia = 0.05,       # minimum value for weigths (see Leisch, 2004)
    thres = 0.01         
    ) {
    
    # Calculate Starting Parameters
    
    n          <- length(dat)
    a          <- min(extr)
    b          <- max(extr)
    Lseries    <- LgL <- LgLc <- NULL
    parmatmax  <- parmatmean <- NULL
    pil        <- array(0,c(g,nx+1))
    theta      <- array(0,c(g,2,nx+1))
    thetak     <- NULL
    
    begin      <- Phi(dat,g,rr=rr,extr=c(a,b),initialization = initial, starting.values = start.values,thre=thres)
    pil[,1]    <- beginpi    <- begin$pil
    theta[,,1] <- begintheta <- begin$theta  # starting values

    if(g == 1){
      fiu <- function(x,dati=dat,aa=a,bb=b)
      {
        tot <- sum(log(betad(dati, cfm(x[1],a1=aa,b1=bb), exp(x[2]),a1=aa,b1=bb)))
        return(-tot)
      }
      theta[g,1,1]  <- mfc(begintheta[1],a1=a,b1=b)
      theta[g,2,1]  <- log(begintheta[2])
      thetak        <- optim(par=theta[g,,1],fn=fiu)$par
      thetak[1]     <- cfm(thetak[1],a1=a,b1=b)
      thetak[2]     <- exp(thetak[2])
      theta         <- thetak
      L0            <- L1 <- Lseries <- sum(log(betad(dat, theta[1], theta[2], a1=a, b1=b)))
      st            <- 1
      tau           <- 1
      
      pil    <- 1
      parmat <- cbind(theta[1],theta[2],1)
      
    }
    else{
      
      C   <- NULL               
      for(z in 1:g){
        C <- cbind(C,betad(dat, theta[z,1,1], theta[z,2,1], a1=a, b1=b))
      }
      B      <- (C %*% pil[,1])^(-1)    
      
      LgL[1]  <- sum(log(1/B))          # Starting Log-Likelihood
      tau    <- pil[,1] * t(C) * (t(B) %x% rep(1,g))
      
      LgLc[1] <- sum(sapply(1:g, function(j)  
        sapply(1:n, function(i)
          tau[j,i] * ( log(pil[j,1]) + log(betad(dat[i], m=theta[j,1,1], h=theta[j,2,1], a1=a, b1=b)) )
               )
                            )
                     )          # Starting Complete Log-Likelihood
      
      # Cicle
      
      st <- 1 # counting number of iterations
      
      for(k in 1:nx) {
        
        # tau previous iteration
        
        extau <- tau
        
        # E-Step
        
        tau <- pil[,k] * t(C) * (t(B) %x% rep(1,g))  # (g x n)
        
        # C-step
        
        if(EMtype=="CEM")
          tau <- (t(matrix(rep(apply(t(tau),1,max),g),n,g))==tau)*1
        
        # S-step
        
        if(EMtype=="SEM")
          tau <- sapply(1:n, function(i) rmultinom(1, size = 1, prob=tau[,i]))
        
        # ------ #
        # M-Step #
        # ------ #
        
        # Updating weights
        
        pik <- rowMeans(tau)
        
        if(EMtype == "CEM" | EMtype == "SEM"){
          
          piktemp <- (pik<soglia)*soglia+(pik>=soglia)*pik
          pik     <- piktemp/sum(piktemp)
          
        }
        
        # Updating parameters
        
        f <-
          function(x,dati=dat,aa=a,bb=b)
          {
            tot <- 0
            for(i in 1:g) {
              tot <- tot + sum(tau[i,] %*% log(betad(dati, cfm(x[2*i-1],a1=aa,b1=bb), exp(x[2*i]),a1=aa,b1=bb)))   
            }   
            return(-tot)
          }
        
        thetatempm  <- mfc(theta[,1,k],a1=a,b1=b)
        thetatempv  <- log(theta[,2,k])
        inter       <- optim(par=c(t(cbind(thetatempm,thetatempv))),fn=f,method = "BFGS")$par
        thetak      <- matrix(inter,g,2,byrow=TRUE)
        thetak[,1]  <- cfm(thetak[,1],a1=a,b1=b)
        thetak[,2]  <- exp(thetak[,2])
        
        # Sostituting old proportions/parameters and Calculating new Likelihood
        
        theta[,,(k+1)] <- thetak
        pil[,(k+1)]    <- pik
        
        C <- NULL
        for(z in 1:g){
          C <- cbind(C,betad(dat, theta[z,1,k+1], theta[z,2,k+1], a1=a, b1=b))
        }
        B <- (C %*% pil[,(k+1)])^(-1)
        
        L1 <- LgL[k+1] <- sum(log(
          sapply(1:n, function(i)
            mixfunbet(dat[i], G=g, parametri=theta[,,k+1], pesi=pil[,k+1], a=a, b=b)
                 )
          ))
        
        L1c <- LgLc[k+1] <-  sum(sapply(1:g, function(j)   # Matrix of quantities necessary to calculate the second part of the complete log-likelihood
          sapply(1:n, function(i)
            tau[j,i] * ( log(pik[j]) + log(betad(dat[i], m=thetak[j,1], h=thetak[j,2], a1=a, b1=b)) )
                 )
                                        )
                                 )
        
        # Convergence condition - Stopping rule
        
        if(k>=2 & EMtype=="EM"){if(abs(LgL[k+1]-LgL[k]) < eps) break}
        
        if(k>=2 & EMtype=="CEM"){if(sum(tau==extau)==n*g) break}
        
        if(k>=2 & EMtype=="SEM"){}
        
        st <- st+1
        print(st)
        
      }
      
      # Order according to the modes
      
      parmat <- cbind(theta[,,(k+1)],pil[,k+1])
      tr     <- order(parmat[,1])
      parmat <- parmat[tr,]
      
      if(EMtype=="SEM"){ # different rules to choose the best SEM-estimates
        
        # SEM-mean
        
        pilmean    <- rowMeans(pil[,max(1,(k-98)):(k+1)])
        thetamean  <- apply(theta[,,max(1,(k-98)):(k+1)],c(1,2),mean)
        parmatmean <- cbind(thetamean,pilmean)
        tr         <- order(parmatmean[,1])
        parmatmean <- parmatmean[tr,]
        
        # SEM-max
        
        pilmax     <- pil[,max(which(LgL==max(LgL)))]
        thetamax   <- theta[,,max(which(LgL==max(LgL)))]
        parmatmax  <- cbind(thetamax,pilmax)
        tr         <- order(parmatmax[,1])
        parmatmax  <- parmatmax[tr,]
        
      }
      
    }
        
    # AIC and BIC
    
    aic      <- -2*L1+2*(3*g-1)
    bic      <- -2*L1+(3*g-1)*log(n)
    minuslog <- -2*L1
    caic     <- -2*L1+(3*g-1)*(log(n)+1)
    
    if(g>1) {
      pil   <- parmat[,3]
      theta <- parmat[,1:2]
    }
    
    return(
      list(
        pil               = pil,        # starting weights
        theta             = theta,      # estimated parameters
        ni                = st,         # number of iterations
        L                 = L1,         # likelihood
        beginpi           = beginpi,    # starting proportion parameters
        begintheta        = begintheta, # starting theta parameters
        LgL               = LgL,        # likelihood at each iteration
        LgLc              = LgLc,       # complete-likelihood of each iteration
        minuslog          = minuslog,
        aic               = aic,
        bic               = bic,
        caic              = caic,
        matricepar        = parmat,     # the last set of estimated parameters if SEM is used
        matriceparSEMmean = parmatmean, 
        matriceparSEMmax  = parmatmax
        )
      )
  }