#########################################################################
## Constraint Function and Inverse Constraint Function for parameter m ##
#########################################################################

cfm <- function(x,a1=0)  a1+exp(x)
mfc <- function(x,a1=0)  log(x-a1)


############################################
## Reparameterized Gamma density function ##
############################################

gammad <-
  function(
    x,            # evaluation points
    m,            # mode parameter 
    h,            # variability parameter (denoted with "v" in the reference paper)
    a1=0,         # lower bound of the support
    logar=FALSE
    ) {
    return(
      dgamma(x-a1, shape=(m-a1)/h+1, scale=h, log=logar)
      )
  }


###########################
## The Mixture  Function ##
###########################

mixfungam <- function(x,          # evaluation points
                      pesi,       # weights 
                      parametri,  # parameters
                      a=0,        # lower bound of the supoprt
                      G,          # number of groups (k in the reference paper)
                      loga=FALSE
                      ) {
  if(G == 1){
    iniz <- gammad(x,parametri[1],parametri[2],a1=a,logar=loga) 
    return(iniz)
  }    
  
  else{
    
    fx   <- sapply(1:G, function(j) gammad(x,parametri[j,1],parametri[j,2],a1=a,logar=loga) )
    iniz <- fx %*% pesi
    return(iniz)
  }
}


################################################################################################
## Given the data, the function "kerder" provides:                                            ##
##           - kernel estimation of the density and of its first and second order derivatives ##
##           - Bump intervals (bumps) and their points of local maximum                       ##   
################################################################################################

kerder <- function(x=0,  # evaluation points
                   data, # observed data
                   h=0   # bandwidth parameter
                   )
{
  
  n <- length(data)
  y <- density(data)
  if(h==0) h <- y$bw          # if h is not given, it is obtained through the function "density()"
  if(x==0) x <- y$x           # if x is not given, it is obtained through the function "density()"
  q <- length(x)
  f <- f1 <- f2 <- numeric(q) # functions are evaluated on all the q points x
  
  # Kernel estimation of the density (f) and its first and second order derivatives (f1 and f2)
  
  for(j in 1:q){
    f[j]  <-  1/(n*h*sqrt(2*pi))*sum(exp(-(x[j]-data)^2/(2*h^2)))
    f1[j] <- -1/(n*h^3*sqrt(2*pi))*sum((x[j]-data)*exp(-(x[j]-data)^2/(2*h^2)))
    f2[j] <-  1/(n*h^2*sqrt(2*pi))*sum(((x[j]-data)^2/h^2-1)*exp(-(x[j]-data)^2/(2*h^2)))
  }
  
  # Find the Bumps
  
  a <- numeric(q-1)
  b <- rep(NA,q-1)
  
  for(j in 1:(q-1)){
    a[j]                  <- approx(f2[j:(j+1)],x[j:(j+1)],0)$y
    if(!is.na(a[j])){b[j] <- 1/(n*h*sqrt(2*pi))*sum(exp(-(a[j]-data)^2/(2*h^2)))}
  }
  
  bump  <- a[which(a!="NA")]  # bump bounds: points on the x-axis where f2=0
  bumpy <- b[which(b!="NA")]  # estimated f in correspondence of the bump bounds
  bump  <- c(-Inf,bump,+Inf)  
  bumpy <- c(0,bumpy,0)        
  l     <- length(bump)        
  
  # Find the maximum value in each Bump
  
  maxim <- maximy <- maximxy <- NULL
  su <- 1
  bumpxy <- NULL
  
  for(i in 1:(l-1)){
    
    posx     <- which(x>=bump[i] & x<=bump[i+1])
    posmod   <- posx[f2[posx]<=0 & f[posx]==max(f[posx])]
    
    if(length(posmod!=0)){  
      maxim[su]  <- x[posmod]                                                    
      maximy[su] <- 1/(n*h*sqrt(2*pi))*sum(exp(-(maxim[su]-data)^2/(2*h^2)))   
      maximxy    <- rbind(maximxy,c(maxim[su],maximy[su]))
      bumpxy     <- rbind(bumpxy,cbind(bump[i:(i+1)],bumpy[i:(i+1)]))            
      su         <- su + 1
    }
    
  }
  
  return(list(f=f,
              f1=f1,
              f2=f2,
              x=x,
              h=h,
              bump=bump,        
              maxim=maxim,      
              bumpxy=bumpxy,    # points defining the bump bounds (first column) and correspondening values of "f"
              maximxy=maximxy   # points of maximum (first column) and maxima (second column) for "f" in all the bump
              )
         )
  
}


###########################################################
## Given the number of mixture components,               ##
## the function "Modgroup" calculates the modes          ##
## and the corresponding values of the estimated density ##
###########################################################

Modgroup <-
  function(
    x,           # vector of data
    g,           # number of components of the gamma mixture
    tol = 0.001, # tollerance for the stopping rule in the cicle for find the bandwidth
    maxit = 100  # maximum number of iterations to find the bandwidth
    ) {
    
    n <- length(x)
    
    h1  <- (max(x)-min(x))/1000
    h2  <- max(x)-min(x)
    
    # Algorithm to compute the bandwidth 
    
    for(i in 1:maxit){         
      hprov <- (h1+h2)/2
      inter <- kerder(data=x,h=hprov)
      gmean <- length(inter$maxim)
      
      if(gmean<=g) h2 <- hprov
      else         h1 <- hprov
      
      if(gmean==g & (h2-h1)/h1<tol) break
    }
    
    # Assigning bumps and modes according to the function kerder()
    
    hp         <- hprov
    modes      <- inter$maxim
    
    bump       <- inter$bump
    bumpxy     <- inter$bumpxy
    maximxy    <- inter$maximxy        
    
    return(list(
      modes = modes,      
      hp=hp,
      bump=bump,
      bumpxy=bumpxy,
      maximxy=maximxy
      )
           )
  }


###################################################################
## "Phi" calculates the starting parameters for the EM algorithm ##
###################################################################

Phi <-
function(
    x,                          # vector of data
    g,                          # number of components of the gamma mixture
    extr = 0,                   # left bound of the support of the gamma components
    initialization = "random",  # inizialization technique: "random", "BH" (Bump Hunting), "manual"
    rr = 1,                     # number of replications in random initialization
    starting.values = NULL,     # manual inizialization (list with one vector of weights and one matrix of parameters to provide if initialization = "manual")
    maxit = 100,                # see the Modgroup() function
    tol = 0.001,                # see the Modgroup() function
    thre = 0.008
) {

dati   <- x
n      <- length(x)
a      <- extr
weight <- NULL         
param  <- matrix(,g,2) # m and v parameters of the gamma mixture components
h      <- 0


if(initialization == "random"){
  
  prop    <- weight <- matrix(0,g,rr)
  Loglik  <- NULL
  paramet <- array(0,c(g,2,rr))
  
  for(l in 1:rr) {
    
    ## Weights ##
    
    prop[,l]   <- runif(g,0.2,1)
    weight[,l] <- prop[,l]/sum(prop[,l])
    
    ## Parameters ##
    
    paramet[,1,l] <- runif(g,a+0.1,max(x)*0.9)
    paramet[,2,l] <- runif(g,0.5,1)
    
    Loglik[l] <- sum(log(
      sapply(1:n, function(i)
        mixfungam(dati[i], G=g, parametri=paramet[,,l], pesi=weight[,l], a=a)
             )
      ))
    
  }
  
  ind       <- which(Loglik==max(Loglik))
  weight    <- weight[,ind]
  param     <- paramet[,,ind]
}


if(initialization == "manual"){

    weight <- starting.values[[1]] # numeric vector of size g
    param  <- starting.values[[2]] # (g x 2) matrix: m on the first column and v on the second one

}


if(initialization == "BH"){

  G <- means <- variances <- acca <- accam <- weight <- NULL
  y <- sort(x)

  if(g == 1){   group <- Modgroup(x,g=g)
                h <- group$hp

                m <- group$maximxy[1,1]
             
		            fiu <- function(x) {
                  fi <- 0
                  for(i in 1:n) {fi <- log(gammad(dati[i],m,x)) +fi}
                  return(-fi)
                }
	
                acca   <- optimize(f=fiu,interval=c(0,1000))$minimum
                param  <- cbind(m,acca)
                weight <- 1
              }

  else{

  group <- Modgroup(x,g=g)
  h     <- group$hp
  m     <- group$maximxy[,1]
  int   <- c(a,(m[1:(g-1)]+m[2:g])/2,+Inf)

  for(j in 1:g){
      G[[j]]    <- y[which(y>=int[j] & y<int[j+1])]# generating g group in the list G
      weight[j] <- length(G[[j]])/n
      ng        <- length(G[[j]])
      
      foia <- function(x,para=m,aaa=a) {
                logver    <- 0
                for(i in 1:ng){
                    logver <- log(gammad(G[[j]][i],m[j],x,a1=aaa)) + logver
                  }      
                return(-logver)                
                }

      acca[j] <- optimize(f=foia,interval=c(thre,1))$minimum
  
    }

  param <- cbind(m,acca)

  }
}

return(
       list(
       pil = weight,  # vector of dimension g containing estimated weights
       theta = param, # (g x 2) matrix containing the estimated component parameters
       h=h
       )
      )
}


###################
## Gamma Mixture ##
###################

gammamix <-
function(
    dat,                 # vector of data
    g,                   # number of components of the gamma mixture
    extr = 0,            # left bound of the support of gamma distributions
    nx = 1000,           # maximum number of iterations
    rr = 1,              # number of random initializations
    eps = 0.00001,       # convergence limit
    EMtype = "EM",       # EM-based variant: "EM", "CEM", "SEM"
    initial = "random",  # inizialization technique: "random", "BH", "manual"
    start.values = NULL, # manual inizialization (see Phi() function)
    thres = 0.008,       # see the Phi() function
    sogliav = 0.05,       # minimum value for parameters v
    sogliapi = 0.05      # minimum value for weigths (see Leisch, 2004)
) {

# Calculate Starting Parameter

n          <- length(dat)
a          <- extr
Lseries    <- LgL <- LgLc <- NULL
parmatmax  <- parmatmean <- NULL
pil        <- array(0,c(g,nx+1))
theta      <- array(0,c(g,2,nx+1))
thetak     <- NULL

begin      <- Phi(dat,g,extr=a,initialization = initial, starting.values = start.values, thre = thres)
pil[,1]    <- beginpi    <- begin$pil
theta[,,1] <- begintheta <- begin$theta  # starting values

sogliam     <- max(dat)
theta[,1,1] <- (theta[,1,1]>sogliam)*sogliam+(theta[,1,1]<sogliam)*theta[,1,1]

if(g == 1){
  fiu <- function(x,dati=dat,aa=a)
  {
    tot <- sum(log(gammad(dati, cfm(x[1],a1=aa), exp(x[2]),a1=aa)))
    return(-tot)
  }
  theta[g,1,1]  <- mfc(begintheta[1],a1=a)
  theta[g,2,1]  <- log(begintheta[2])
  thetak        <- optim(par=theta[g,,1],fn=fiu)$par
  thetak[1]     <- cfm(thetak[1],a1=a)
  thetak[2]     <- exp(thetak[2])
  theta         <- thetak
  L0            <- L1 <- Lseries <- sum(log(gammad(dat, theta[1], theta[2], a1=a)))
  st            <- 1
  tau           <- 1
  
  pil    <- 1
  parmat <- cbind(theta[1],theta[2],1)
  
}

else{

  C       <- NULL                   
  for(z in 1:g){
	C <- cbind(C,gammad(dat, theta[z,1,1], theta[z,2,1], a1=a))
  }
  
  #pippo <- 20
  
  #C <- (C<10^(-pippo))*10^(-pippo)+(C>10^(-pippo))*C # problem with R
  
  B       <- (C %*% pil[,1])^(-1)   

  LgL[1]  <- sum(log(1/B))          # Starting Log-Likelihood
  tau     <- pil[,1] * t(C) * (t(B) %x% rep(1,g))

  LgLc[1] <- sum(sapply(1:g, function(j)  
                          sapply(1:n, function(i)
                            tau[j,i] * ( log(pil[j,1]) + gammad(dat[i], m=theta[j,1,1], h=theta[j,2,1], a1=a, logar=TRUE) )
                            )
                          )
                      )          # Starting Complete Log-Likelihood

   
  # Cicle

  st <- 1 # counting number of iterations

  for(k in 1:nx) {

    # tau previous iteration

      extau <- tau

    # E-Step

    tau <- pil[,k] * t(C) * (t(B) %x% rep(1,g))  # (g x n)
      
    # C-step
    
    if(EMtype=="CEM")
      tau <- (t(matrix(rep(apply(t(tau),1,max),g),n,g))==tau)*1
    
    # S-step
    
    if(EMtype=="SEM")
  	  tau <- sapply(1:n, function(i) rmultinom(1, size = 1, prob=tau[,i]))
    
    # ------ #
    # M-Step #
    # ------ #

    # Updating weights

    pik <- rowMeans(tau)

    if(EMtype == "CEM" | EMtype == "SEM"){

  	  soglia  <- sogliapi
  	  piktemp <- (pik<soglia)*soglia+(pik>=soglia)*pik
      pik     <- piktemp/sum(piktemp)
     
  	  }

    # Updating parameters

  	f <- function(x,dati = dat, aa = a){
            tot <- 0
            for(i in 1:g) {
            tot <- tot + sum(tau[i,] %*% gammad(dati, cfm(x[2*i-1],a1=aa), exp(x[2*i]), a1=aa, logar=TRUE))
            }
          return(-tot)
          }

    thetatempm  <- mfc(theta[,1,k],a1=a)
    thetatempv  <- log(theta[,2,k])
    inter       <- optim(par=c(t(cbind(thetatempm,thetatempv))),fn=f)$par
    thetak      <- matrix(inter,g,2,byrow=TRUE)
    thetak[,1]  <- cfm(thetak[,1],a1=a)
    thetak[,2]  <- (exp(thetak[,2])<sogliav)*sogliav+(exp(thetak[,2])>=sogliav)*exp(thetak[,2])

    # Sostituting old proportions/parameters and Calculating new Likelihood

    
    thetak[,1] <- (thetak[,1]>sogliam)*sogliam+(thetak[,1]<sogliam)*thetak[,1]
      
    theta[,,(k+1)] <- thetak
    pil[,(k+1)]    <- pik
    
    C <- NULL
    for(z in 1:g){
  	C <- cbind(C,gammad(dat, theta[z,1,k+1], theta[z,2,k+1], a1=a))
  	}
    
    #C <- (C<10^(-pippo))*10^(-pippo)+(C>10^(-pippo))*C # problem with R
     
    B <- (C %*% pil[,(k+1)])^(-1)
  
    L1 <- LgL[k+1] <- sum(log(
          sapply(1:n, function(i)
            mixfungam(dat[i], G=g, parametri=theta[,,k+1], pesi=pil[,k+1], a=a)
            )
          ))
          
    L1c <- LgLc[k+1] <-  sum(sapply(1:g, function(j)   # Matrix of quantities necessary to calculate the second part of the complete log-likelihood
                          sapply(1:n, function(i)
                            tau[j,i] * ( log(pik[j]) + log(gammad(dat[i], m=thetak[j,1], h=thetak[j,2], a1=a)) )
                            )
                          )
                      )
    
      #print(min(gammad(dat,m=thetak[1,1], h=thetak[1,2],)))
      #print(min(gammad(dat,m=thetak[2,1], h=thetak[2,2],)))
      #print(min(gammad(dat,m=thetak[3,1], h=thetak[3,2],)))
      
      
    # Convergence condition - Stopping rule

    if(k>=2 & EMtype=="EM"){if(abs(LgL[k+1]-LgL[k]) < eps) break}

    if(k>=2 & EMtype=="CEM"){if(sum(tau==extau)==n*g) break}

    if(k>=2 & EMtype=="SEM"){}
  
    st <- st+1
    print(st)
 
  }

  # Order according to the modes

  parmat <- cbind(theta[,,(k+1)],pil[,k+1])
  tr     <- order(parmat[,1])
  parmat <- parmat[tr,]

  if(EMtype=="SEM"){
	
  	# SEM-mean

  	pilmean    <- rowMeans(pil[,max(1,(k-98)):(k+1)])
  	thetamean  <- apply(theta[,,max(1,(k-98)):(k+1)],c(1,2),mean)
  	parmatmean <- cbind(thetamean,pilmean)
  	tr         <- order(parmatmean[,1])
  	parmatmean <- parmatmean[tr,]

  	# SEM-max
  
  	pilmax     <- pil[,max(which(LgL==max(LgL)))]
  	thetamax   <- theta[,,max(which(LgL==max(LgL)))]
  	parmatmax  <- cbind(thetamax,pilmax)
  	tr         <- order(parmatmax[,1])
  	parmatmax  <- parmatmax[tr,]

  }

}

# AIC and BIC

aic      <- -2*L1+2*(3*g-1)
bic      <- -2*L1+(3*g-1)*log(n)
minuslog <- -2*L1
caic     <- -2*L1+(3*g-1)*(log(n)+1)

if(g>1) {
        pil   <- parmat[,3]
        theta <- parmat[,1:2]
        }

return(
       list(
       pil               = pil,        # estimated weights
       theta             = theta,      # estimated parameters
       ni                = st,         # number of iterations
       L                 = L1,         # likelihood
       beginpi           = beginpi,    # starting weights
       begintheta        = begintheta, # starting parameters
       LgL               = LgL,        # likelihood at each iteration
       LgLc              = LgLc,       # complete-likelihood of each iteration
       minuslog          = minuslog,
       aic               = aic,
       bic               = bic,
       caic              = caic,
       matricepar        = parmat,     # the last set of estimated parameters if SEM is used
	     matriceparSEMmean = parmatmean,
	     matriceparSEMmax  = parmatmax
       )
      )
}